!* ---------------------------------------------------------------- SMC2PSAGMRot
      Program SMC2PSAGMRot

! Reads an SMC filename from a control file, loops over a series of rotation 
! angles, for each computing the geometric mean of the two horizontal
! components.  This version prints out the PSA of each individual
! as-recorded component, the gmean of the two as-recorded components, the
! maximum and minimum PSA over all angles of rotation, GMRotD50, GMRotI50,
! and pga, pgd corresponding to the angle of GMRotI50.  Also
! print out the angles for GMROTD50.
! The program now prints out the minimum and maximum gmean over all angles.
!
 
! Structure of control file:
! Line 1 follows:
!!Control file for program SMC2PSAGMRot
!!Summary file name:
! gmeanrot.sum
!!d_rot_angle
!  0.5  
!!Use PEER NGA periods for gmrot50 computation (yfull/ysub/n):
! ysub
!!if "n", then specify periods: nperiods log spaced from per_low to per_high:
!!need these as placeholders, even if use NGA periods
! 200 0.01 90
!!Damping value (damp):
! 0.05
!!t4mean (removes a mean from the whole record, where the mean is
!!    determined from t = 0 to t = t4mean.  If t4mean = 0.0,
!!    the mean of the whole record is used, and if t4mean < 0, no mean is
!!    removed.  Note that if working with a processed record, it is
!!    inadvisible to remove the mean as this may produce unintended
!!    distortions at long periods --- so in most cases set t4mean < 0.0):
! -10.0  
!! Skip analysis (including rotation) if npts differ by more than:
!  2
!! Extension for output file (append to end of first file name)
! .gmrot90.asc
!! tmax to be used in the computations
!  10.0
!! list of file names (assumes pairs of files) 
!    0516a.smc
!    0516c.smc
!    0526a.smc
!    0526c.smc
!  StoP

! Sign conventions:  the azimuth of the radial component is 180+azimuth from
! station to reference point, and the transverse component is this radial 
! azimuth plus 90 degrees.  This should be equivalent to the convention
! in Aki and Richards, Fig. 4.10, p. 96.

! This is incomplete, because the program will eventually also deal with case
! for which i_spcfy_fname = 1.  A program that works 
! with this is smcrtold (the old version of smc_rot)

! Dates: 06/18/05 - Written by D. Boore, based on gmeanrot.  
!        07/03-05 - Add pga. pgv to output.  Also, decreased the dimensions
!                   of gmean from (200,300) to (110,100)
!        07/04/05 - Incorporate Paul Spudich's idea of computing the 
!                   oscillator time series for the as-recorded motions
!                   and rotating these to find PSA for the various angles.
!        07/04/05 - Remove max_dimen from calling list of rotate
!        09/21/05 - Changed azimuth specification (dazm, azm_low, azm_high),
!                   one reason to be sure to avoid computations for azm = 0.0
!                   90.0, which are the same
!        09/21/05 - Add SD to output
!        09/21/05 - Make name of penalty function more specific, add 
!                   column of azm relative to azm of component one.
!        10/09/05 - Define mean as mean of values if nrot
!                   is even.
!        10/10/05 - Specify 90 as upper limit, but program decides nrot
!                   so as not to have rots of 0 and 90 (by doing this,
!                   user need not be sure that rotmax = 89.5 if
!                   drot = 0.5 and 89.0 if drot = 1.0, etc).
!        02/08/06 - Added character string now must include "." at 
!                   beginning if wanted; the program does not add it
!                   as before.
!        02/08/06 - Change dimensions of gmean to allow more periods, 
!                   fewer angles
!        02/08/06 - Changed name of variable gm50_indper to gmrotd50, because
!                   "ind" in the original name, which stood for "individual"
!                   would likely be interpreted as "independent".
!        02/11/06 - Fix using thigh for nga periods.
!        02/12/06 - Normalize penalty sum by Nper
!        10/17/07 - Transparent change to user needed because smcread can
!                   read two input formats
!        10/18/07 - Written by D. M. Boore, based on GMRot50.
!        10/18/07 - Don't use azimuths in headers (which are often
!                   missing); instead, arbitrarily adding 0.0 and 90.0
!                   to the two components.
!        07/08/08 - Print out the minimum and maximum gmean over all angles.
!        07/14/08 - Check for odd or even nazm and adjust computation of gmrot50
!        07/14/08 - Print rotation angle for the minimum of the penalty function
!        07/14/08 - Read a value of tmax and then a list of files (this is more useful than
!                   the previous version, in which each pair of files was followed by tmax.
!                   If want this capability, then in a future version I can do what
!                   blpadflt does:  read some processing parameters, the file list, and
!                   then if desired another set of processing parameters, a file list, etc.)
!        07/15/08 - A number of changes: 
!                   1) assume rotations will be from 0 to an angle just less than 90.0 
!                      (so only need to specify drotangle in control file).
!                   2) change from emphasis on azimuths to rotations (I do not care about
!                      the actual component azimuth for this application).
!                   3) correct previous error that stop one rotation short of what should
!                      have been used.
!        04/17/09 - Write out sd for the as recorded spectra
!        06/25/09 - Use MOD to find out if nrot is odd or even
!                   (after Chapman, Fortran 90/95, 2nd Ed., p. 307)
!                 - Remove this revision, because n50 was not defined.  I should rethink
!                   the use of n50.
!        10/15/09 - Set nosc_ts (used in rotations) to minimum value of the npts for each time series 
!                   (and subtract 1: see rdcalcts).  
!        12/04/15 - Bring in subroutines by including the file smc2psa_util_subs.for
!                 - Replace call to rdcalcts with one to rdrvaa_rd_ts
!        02/28/18 - Replace call to rdrvaa_rd_ts with one to rdrvaa_rd_rv_ts               
  
      character char_head(11,2)*80, comments(200,2)*80
      character cmnts2skip(20)*80, path_in*40, path_out*40      
       
      integer idate_time_values(8)
      
      real time_sec_start, time_sec_stop

      integer int_head(48,2), nc_f_in(2),  nc_f_out, nc_rc(2)
      integer npts(2)

      real real_head(50,2), work(400), rot_angle(400)
      real alat_sta(2), along_sta(2)
      real a_in(:,:), a_out(:,:), vel(:), dis(:)
      real osc_ts_as_recorded(:,:), osc_ts_rotated(:,:), dum(:)
      allocatable :: a_in, a_out, vel, dis 
      allocatable ::  osc_ts_as_recorded, osc_ts_rotated, dum
      
      real period(300), rd(2), psa(2), pga(2), pgv(2), 
     :    gmean(300,400), gmean_norm(300,400),
     :    gmrotd50(400), penalty(400), gm50(400),
     :    gmrotd00(400), gmrotd100(400)
      real psa_1_ar(400), psa_2_ar(400), psa_larger_ar(400), 
     :      psa_gm_ar(400)
      real psamaxrot(400), psaminrot(400)
      real angle4psamax(400), angle4psamin(400)
      real omega, omega2
      real per4az(10)
      integer ndx4az(10)
         

      real rnull(2), azm_in(2), azm_out(2), azm4file(200), sps(2)
      character buf_c*80, buf4_c*4, f_ctl*80, f_sum*80,
     :          buf_out*79
      character  f_in(2)*80, f_out*100, f_ext*80, f_penalty*100,
     :           c_az*7, rc(2)*12
      character per4az_c*5, colhead1(10)*12, colhead2(10)*12, frmt_c*80
      character use_nga_per4gm*10
      logical f_ctl_exist
      logical rmv_trnd, odd_l
      
      real per_nga_full(107), per_nga_sub(23)   ! include pgv and pga as per = -1.0, 0.0
      data per_nga_full/
     :  -1.0, 0.0,   
     :   0.010,0.020,0.022,0.025,0.029,0.030,0.032,0.035,0.036,0.040,
     :   0.042,0.044,0.045,0.046,0.048,0.050,0.055,0.060,0.065,0.067,
     :   0.070,0.075,0.080,0.085,0.090,0.095,0.100,0.110,0.120,0.130,
     :   0.133,0.140,0.150,0.160,0.170,0.180,0.190,0.200,0.220,0.240,
     :   0.250,0.260,0.280,0.290,0.300,0.320,0.340,0.350,0.360,0.380,
     :   0.400,0.420,0.440,0.450,0.460,0.480,0.500,0.550,0.600,0.650,
     :   0.667,0.700,0.750,0.800,0.850,0.900,0.950,1.000,1.100,1.200,
     :   1.300,1.400,1.500,1.600,1.700,1.800,1.900,2.000,2.200,2.400,
     :   2.500,2.600,2.800,3.000,3.200,3.400,3.500,3.600,3.800,4.000,
     :   4.200,4.400,4.600,4.800,5.000,5.500,6.000,6.500,7.000,7.500,
     :   8.000,8.500,9.000,9.500,10.000/
      nper_nga_full = 107 
      
        
      data per_nga_sub/
     :   -1.000,
     :    0.000,
     :    0.010,
     :    0.020,
     :    0.030,
     :    0.050,
     :    0.075,
     :    0.100,
     :    0.150,
     :    0.200,
     :    0.250,
     :    0.300,
     :    0.400,
     :    0.500,
     :    0.750,
     :    1.000,
     :    1.500,
     :    2.000,
     :    3.000,
     :    4.000,
     :    5.000,
     :    7.500,
     :   10.000/
      nper_nga_sub = 23 


      pi = 4.0*atan(1.0)
      twopi = 2.0*pi
      d2r = pi/180.0
      nper_dimen = 300
      nrot_dimen = 400

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = smc2psagmrot.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'smc2psagmrot.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip, nc_comments)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call get_lun(nu_sum)
      call trim_c(f_sum,nc_f_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')    
      write(nu_sum,'(a)') ' Summary file for program GMeanRot'
      write(nu_sum,'(2a)') '   Using control file: ', f_ctl
      do i = 1, nc_comments
        write(nu_sum,'(a)') cmnts2skip(i)
      end do
      write(nu_sum,'(a)') f_sum

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) d_rot_angle
      rot_low = 0.0
      rot_high = 0.0
      nrot = 1
      do while (rot_high .lt. 90.0)
        rot_high = rot_high + d_rot_angle
        nrot = nrot + 1
      end do
      
      nrot = nrot - 1
      rot_high = rot_high - d_rot_angle
      
      print *,' d_rot_angle, nrot, rot_high, rot_high_by_comp = ', 
     :         d_rot_angle, nrot, rot_high, float(nrot - 1)*d_rot_angle
      write(nu_sum,'(a)') 
     :  ' d_rot_angle, nrot, rot_high, rot_high_by_comp = ' 
      write(nu_sum,*) 
     :    d_rot_angle, nrot, rot_high, float(nrot - 1)*d_rot_angle
      if (nrot .gt. nrot_dimen) then
        nrot = nrot_dimen
        d_rot_angle = 90.0/float(nrot)
        write(nu_sum,'(a,1x,i3)') ' resetting nrot to ', nrot
        write(nu_sum,'(a,1x,i3)') ' and d_rot_angle to ', d_rot_angle
      else        
        write(nu_sum,'(a,1x,i3)') ' nrot = ', nrot
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      use_nga_per4gm = ' '
      read(nu_ctl,'(a)') use_nga_per4gm
      call upstr(use_nga_per4gm)
      call trim_c(use_nga_per4gm,nc_use_nga_per4gm)
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) nper_specify, per_low, per_high
      write(nu_sum,*) nper_specify, per_low, per_high
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) damp
      write(nu_sum,*) damp
      
       
      nd2 = int(float(nrot)/2.0)
      if (nrot - 2*nd2 .ne. 0) then
        n50 = nd2 + 1  ! odd
        odd_l = .true.
      else
        n50 = nd2      ! even
        odd_l = .false.
      end if

!      if (mod(nrot,2) == 0) then  ! even
!        odd_l = .false.
!      else
!        odd_l = .true.
!      end if
        
         
      print *,' n50, odd_l: ', n50, odd_l  
      write(*,'(a)') ' n50 ='
      write(*,*)       n50
      
      write(nu_sum,'(a)') ' n50 ='
      write(nu_sum,*)       n50
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) t4mean_in
      write(nu_sum,*) t4mean_in
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) npts_skip
      write(nu_sum,*) npts_skip
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      f_ext = ' '
      read(nu_ctl,'(a)') f_ext
      call trim_c(f_ext,nc_f_ext)
      write(nu_sum,'(a)') f_ext(1:nc_f_ext)
         
! Set up periods:
        


      if(use_nga_per4gm(1:nc_use_nga_per4gm) .eq. 'YFULL') then
        nper_max = nper_nga_full - 2
        do i = 1, nper_max
          period(i) = per_nga_full(i+2)
        end do
      else if (use_nga_per4gm(1:nc_use_nga_per4gm) .eq. 'YSUB') then
        nper_max = nper_nga_sub - 2
        do i = 1, nper_max
          period(i) = per_nga_sub(i+2)
        end do
      else
        nper_max = nper_specify
        if (nper_max .gt. nper_dimen) then
          nper_max = nper_dimen
          nper_specify = nper_dimen
          write(nu_sum,'(a,1x,i3)') 
     :          ' Resetting nper_max to ', nper_max
          write(nu_sum,'(a,1x,i3)') 
     :          ' Resetting nper_specify to ', nper_specify
        end if
        dlogt = alog10(per_high/per_low)/float(nper_specify-1)
        do i = 1, nper_max
          period(i) = per_low*10.0**(float(i-1) * dlogt)
        end do   
      end if
      


      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      write(nu_sum, '(a)') ' Just before 1000 continue '

      read(nu_ctl,*) thigh
      write(nu_sum,'(1x,a,1x,f6.2)') ' thigh = ', thigh
      write(nu_sum,'(1x,a,1x,i3)') ' nper_max = ', nper_max

! Set nperiods:

      do i = 1, nper_max
        write(nu_sum,'(a,1x,i3, 1x,f8.4,1x,f6.2)') 
     :        ' i, period, thigh = ', i, period(i), thigh
        if(period(i) .gt. thigh) then
          nperiods = i-1
          go to 876
        end if
      end do
      nperiods = nper_max
876   continue          
      write(nu_sum,'(a,1x,i3, 1x,f8.4,1x,f6.2)') 
     :        ' nperiods, period(nperiods), thigh = ',
     :          nperiods, period(nperiods), thigh
       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

1000  continue ! Loop over files

        write(nu_sum,*)

 
! First get max dimension and allocate arrays:
        do j = 1, 2
          buf_c = ' '
          read(nu_ctl,'(a)') buf_c
          call trim_c(buf_c,nc_buf)
          buf4_c = ' '
          buf4_c = buf_c(1:4)
          call upstr(buf4_c)

          if (buf4_c(1:4) .eq. 'STOP') then
            write(*, '(a)') 
     :           ' buf_c = '//buf_c(1:nc_buf)//'; QUITTING'
            close(unit=nu_sum)
            close(nu_ctl)
            stop
          end if

          call date_and_time(values=idate_time_values)
          time_sec_start = float(idate_time_values(5)*60*60+
     :                           idate_time_values(6)*60   +
     :                           idate_time_values(7))     +
     :                     float(idate_time_values(8))/1000.0
     
          f_in(j) = ' '
          f_in(j) = buf_c(1:nc_buf)
          call trim_c(f_in(j),nc_f_in(j))
        
          write(nu_sum,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          write(*,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          call smc_npts(f_in(j)(1:nc_f_in(j)), npts(j), sps(j))
        end do
        
        max_dimen = max0( npts(1), npts(2) )
        allocate( a_in(max_dimen,2), a_out(max_dimen,2) )
        allocate( vel(max_dimen), dis(max_dimen) )
        allocate( osc_ts_as_recorded(max_dimen,2) )
        allocate( osc_ts_rotated(max_dimen,2) )
        allocate( dum(max_dimen) )

! Second, loop over input files, reading data:
        do j = 1, 2
          call SMCRead(f_in(j)(1:nc_f_in(j)), 0.0, 100000.0,
     :                 a_in(1,j), char_head(1,j), 
     :                 int_head(1,j), real_head(1,j), 
     :                 comments(1,j))

          npts(j)  = int_head(17,j)  ! should check equality of both time series
          rnull(j) = real_head(1,j)
          azm_in(j) = real(int_head(14,j))
          
          if (j .eq. 1) then   ! overwrite any existing azimuth in the header with 0 and 90
            azm_in(j) = 0.0
          else
            azm_in(j) = 90.0
          end if
          
          azm_out(j) = azm_in(j)  ! needed because Rotate overwrites azm_out
                                  ! in argument list

          do i = 1, npts(j)
            a_out(i,j) = a_in(i,j)
          end do
        end do

! Check for equality of sps:
        if ( sps(1) .ne. sps(2)) then
            write(nu_sum,'(a,1p, 2(1x,e12.5))') 
     :      ' SKIPPING: sps1 .ne. sps2; sps1, sps2 = ', 
     :        sps(1), sps(2)
            go to 1000
        end if
! Check for equality of npts:
        if ( iabs(npts(1) - npts(2)) .gt. npts_skip) then
            write(nu_sum,'(a,3(1x,i6))') 
     :      ' SKIPPING: iabs(npts1 - npts2) > '//
     :      'npts_skip; npts1, npts2, npts_skip = ', 
     :        npts(1), npts(2), npts_skip
            go to 1000
        end if
! Check for equality of rnull:
        if ( rnull(1) .ne. rnull(2)) then
          write(nu_sum,'(a,1p2(1x,e12.5))') 
     :      ' SKIPPING: rnull1 .ne. rnull2; rnull1, rnull2 = ', 
     :        rnull(1), rnull(2)
            go to 1000
        end if
! Check for azimuths being 90 degrees apart (compute the dot product):
        dot = cos(d2r*azm_in(1))*cos(d2r*azm_in(2))+
     :        sin(d2r*azm_in(1))*sin(d2r*azm_in(2))
        eps = 0.001
        if (abs(dot) .gt. eps) then
          write(*,'(a)') '  Dot product not 0.0; see SUM file'//
     :                   ' for details; SKIPPING!!!'
          write(nu_sum,'(a)') '  azm_in(1,2), dot = '
          write(nu_sum,*) azm_in(1), azm_in(2), dot
            go to 1000
        end if

! Obtain coordinates of stations from files and check for equality:
        do j = 1, 2
          alat_sta(j) = real_head(11,j)          
          along_sta(j) = real_head(12,j)          
        end do
        if (alat_sta(1) .ne. alat_sta(2) .or. 
     :      along_sta(1) .ne. along_sta(2) ) then
          write(*,'(a)') '  Station coordinates not equal, '//
     :                   ' see SUM file; SKIPPING!!!'
          write(nu_sum,'(a)') ' alat_sta(1), alat_sta(2) = '
          write(nu_sum,*)       alat_sta(1), alat_sta(2)
          write(nu_sum,'(a)') ' along_sta(1), along_sta(2) = '
          write(nu_sum,*)       along_sta(1), along_sta(2)
            go to 1000
        end if
        
! Passed all equality tests.

      
! Step: Remove a mean from the whole record

!  Determine the appropriate mean:
        do j = 1, 2
          t4mean = t4mean_in
          if(t4mean .lt. 0.0) then  ! do not remove a mean
            avg = 0.0
          else
            tend = float(npts(j)-1)/sps(j)
            if (t4mean .eq. 0.0 .or. t4mean .gt. tend) then   
              t4mean = tend  ! use whole record
            end if
          end if
          n_t4mean = sps(j) * t4mean + 1.1
          cum_avg = 0.0
          do i = 1, n_t4mean
            cum_avg = cum_avg + dble(a_in(i,j))
          end do
          avg = cum_avg/float(n_t4mean)
!          avg = sngl(cum_avg/n_t4mean)
!  Remove the mean from the whole record:
          do i = 1, npts(j)
            a_in(i,j) = a_in(i,j) - avg
          end do
        end do
 
! Step: Loop over periods:

        do iper = 1, nperiods
        
! Compute as-recorded oscillator time series for this period:
          omega = twopi/period(iper)
          omega2 = omega**2
          do j = 1, 2
            dt = 1.0/sps(j)
            d0 = 0.0
            v0 = 0.0
            call rdrvaa_rd_rv_ts(a_in(1,j), npts(j), omega, damp, dt, 
     :                    rd(j), rv, aa, 
     :                    d0, v0, osc_ts_as_recorded(1,j), dum )
!            nosc_ts = npts(j) - 1  ! see rdcalcts (now rdrvaa_rd_ts and now rdrvaa_rd_rv_ts)
          end do
          
! set nosc_ts (used in rotations) to minimum value of the npts for each time series 
! (and subtract 1: see rdcalcts).  If not done, the value of osc_ts_as_recorded(i,j) used 
! in computing osc_ts_rotated(i,j) below could be a mystery value if i exceeds the actual
! length of the jth time series.  Previously nosc-ts was set to npts(j) in the loop above,
! so that nosc_ts after the loop equalled npts(2).  If npts(2) > npts(1) there could be a
! problem (this may have produced some strange results that cropped up only rarely, when
! drotd50 was huge for a particular record when it was included in a long list of
! records, but not when the record as placed near the beginning of the control file).

          nosc_ts = min0(npts(1), npts(2)) - 1   
          
          psa_1_ar(iper) = omega2*rd(1)
          psa_2_ar(iper) = omega2*rd(2)
          psa_gm_ar(iper) = sqrt(psa_1_ar(iper)*psa_2_ar(iper))
          if (psa_1_ar(iper) .gt. psa_2_ar(iper)) then
             psa_larger_ar(iper) = psa_1_ar(iper)
          else
             psa_larger_ar(iper) = psa_2_ar(iper)
          end if
 
! Step: Loop over azimuths:

          psamaxrot(iper) = 0.0
          psaminrot(iper) = 99999.0

          do iz = 1, nrot
        
            rot_angle(iz) = float(iz-1)*d_rot_angle   ! note: iz = 1 corresponds to no rotation
         
            do j = 1, 2
              do i = 1, nosc_ts
                osc_ts_rotated(i,j) = osc_ts_as_recorded(i,j)
              end do
            end do
 
! To decrease possible confusion, I am "fooling" rotate by hardwiring the azimuths
! of components 1 and 2---for this application the actual azimuth are not important.
            azm1 = 0.0
            azm2 = 90.0
            call Rotate(osc_ts_rotated(1,1),osc_ts_rotated(1,2),
     :           rnull(2),
     :           azm1,azm2,rot_angle(iz),nosc_ts)
     
      
! Compute response spectra:

            buf_out = ' '
            buf_out = '+ Patience! Computing rd for rot_angle = '
            write(buf_out(43:47),'(f5.1)') rot_angle(iz)
            buf_out = buf_out(1:47)//' and period '
            write(buf_out(61:68),'(f8.4)') period(iper) 
            write(*,'(a)') buf_out
            
            do j = 1, 2
              call abs_mnmaxidx(osc_ts_rotated(1,j),1,nosc_ts,1,
     :                    ymin,ymax,indx_min,indx_max, rd(j)) 
              psa(j) = omega2 * rd(j) 
              if (psa(j) .gt. psamaxrot(iper)) then
                psamaxrot(iper) = psa(j)
                angle4psamax(iper) = rot_angle(iz)
              end if
              if (psa(j) .lt. psaminrot(iper)) then
                psaminrot(iper) = psa(j)
                angle4psamin(iper) = rot_angle(iz)
              end if
            end do

            gmean(iper, iz) = sqrt(psa(1) * psa(2))
            
          end do  ! end loop over azimuths
          
        end do ! loop over nperiods
        
      
! Now compute stuff

        
        do iper = 1, nperiods
      
          do iz = 1, nrot
            work(iz) =  gmean(iper, iz)
          end do
      
          gmrotd00(iper) = select(1, nrot, work)
          gmrotd100(iper) = select(nrot, nrot, work)
          
          psagm50 = select(n50, nrot, work)          
          if (odd_l) then
            gmrotd50(iper) =  psagm50   !ODD
            write(nu_sum,'(a, 1x,f7.3, 1x,i3, 1x,i3, 1p, 5(1x,e12.5))') 
     :       ' ODD: per, nrot, n50, psagm50n50, gmavg =',
     :      period(iper), nrot, n50, psagm50, gmrotd50(iper),
     :      gmrotd00(iper), gmrotd100(iper)
          else
            psagm50p1 = select(n50+1, nrot, work) !EVEN           
            gmrotd50(iper) = 0.5 * (psagm50 + psagm50p1)  
            write(nu_sum,'(a, 1x,f7.3, 1x,i3, 1x,i3, 1p, 5(1x,e12.5))') 
     :       ' EVEN: per, nrot, n50, , '//
     :       'psagm50n50, psagm50p1, gmavg =',
     :      period(iper), nrot, n50, 
     :      psagm50, psagm50p1, gmrotd50(iper),
     :      gmrotd00(iper), gmrotd100(iper)
          end if      
 
        end do  ! end loop over periods
        
        do iz = 1, nrot
        
          penalty(iz) = 0.0
          
          do iper = 1, nperiods
            gmean_norm(iper,iz) = gmean(iper,iz)/gmrotd50(iper)
            penalty(iz) = penalty(iz) + 
     :                   (gmean_norm(iper,iz) - 1)**2
          end do
          
          penalty(iz) = penalty(iz)/float(nperiods)
          
        end do
        
! Find minimum        
        call mnmaxidx(penalty,1,nrot,1,
     :                    penalty_min,penalty_max,indx_min,indx_max) 
     
        write(nu_sum,'(a,1x,i4, 1x,f7.2,1p,1x,e10.3)') 
     :    ' indx_min, rot_angle_min, penalty_min = ', indx_min, 
     :      rot_angle(indx_min), penalty_min 
        
! Rotate into minimum azimuth for computation of pga and pgv:
        do j = 1, 2
          write(nu_sum,'(a,i1,a,1x,i6)') ' npts(',j,') = ', npts(j)
          do i = 1, npts(j)
            a_out(i,j) = a_in(i,j)
          end do
        end do
 
        print *,' Before call to Rotate'
        azm1 = 0.0
        azm2 = 90.0
        call Rotate(a_out(1,1),a_out(1,2),rnull(2),
     :    azm1,azm2,rot_angle(indx_min),npts(2))
        print *,' After call to Rotate'

! Get peak motions:
        
        do j=1, 2
          dt = 1.0/sps(j)
          v0 = 0.0
          d0 = 0.0
          rmv_trnd = .false.
          call acc2vd(a_out(1,j), npts(j), dt, rmv_trnd, 
     :         v0, d0, vel, dis)
          call abs_mnmaxidx(vel, 1, npts(j), 1,
     :             vmin, vmax, indxpgv_min, indxpgv_max, pgv(j)) 
 
          call abs_mnmaxidx(a_out(1,j), 1, npts(j), 1,
     :             amin, amax, indxpga_min, indxpga_max, pga(j)) 
        end do
        
        pga_gm50 = sqrt(pga(1)*pga(2))
        pgv_gm50 = sqrt(pgv(1)*pgv(2))

! Now write out the gmroti50, gmrotd50 values:

        f_out = ' '
        f_out = f_in(1)(1:nc_f_in(1))//f_ext(1:nc_f_ext)
        call trim_c(f_out,nc_f_out)
        call get_lun(nu_out)
        open(nu_out,file=f_out(1:nc_f_out),status='unknown')
        write(nu_sum,'(a)') ' Output file = '//f_out(1:nc_f_out)
        
        write(nu_out,'(6x,a, 
     :                 3x,a, 3x,a,
     :                 4x,a, 4x,a,
     :                 1x,a, 2x,a,
     :                 2x,a, 
     :                 3x,a,
     :                 2x,a, 2x,a,
     :                 3x,a, 
     :                 3x,a,
     :                 1x,a, 1x,a,
     :                 1x,a,
     :                 1x,a,1x,a,1x,a,
     :                 1x,a,1x,a,1x,a,
     :                 1x,a, 1x,a, 1x,a,
     :                 1x,a, 1x,a, 1x,a
     :                                   )') 
     :    'per', 
     :    'psa_1_ar', 'psa_2_ar', 
     :    'sd_1_ar', 'sd_2_ar', 
     :    'psa_larger_ar', 'psa_gm_ar',
     :    'sd_larger_ar', 
     :    'sd_gm_ar',
     :    'psamaxrot', 'psaminrot',
     :    'sdmaxrot', 
     :    'sdminrot',
     :    'angle4psamax', 'angle4psamin',
     :    'rot4min',
     :    'psa_gmroti50(cm/s2)', 'psa_gmroti50(g)', 'sd_gmroti50(cm)',
     :    'psa_gmrotd50(cm/s2)', 'psa_gmrotd50(g)', 'sd_gmrotd50(cm)',
     :    'psa_gmrotd00(cm/s2)', 'psa_gmrotd00(g)', 'sd_gmrotd00(cm)',
     :    'psa_gmrotd100(cm/s2)', 'psa_gmrotd100(g)', 'sd_gmrotd100(cm)'
     
        write(nu_out,'(1x,f8.4, 172x, 1p,10x,e10.3,6x,e10.3, 6x,e10.3)') 
     :            per_nga_full(1), pgv_gm50
        write(nu_out,'(1x,f8.4, 172x, 1p,10x,e10.3,6x,e10.3, 6x,e10.3)') 
     :            per_nga_full(2), pga_gm50, pga_gm50/981.0
     
        do iper = 1, nperiods
          psa2sd_factor = (period(iper)/(2.0*pi))**2
          sd = gmean(iper,indx_min) * psa2sd_factor
          sd_gmrotd50 = gmrotd50(iper) * psa2sd_factor
          sd_gmrotd00 = gmrotd00(iper) * psa2sd_factor
          sd_gmrotd100 = gmrotd100(iper) * psa2sd_factor
          write(nu_out,'(1x,f8.4, 
     :               1p,
     :                   1x,e10.3, 1x,e10.3,
     :                   1x,e10.3, 1x,e10.3,
     :                   4x,e10.3, 1x,e10.3,
     :                   4x,e10.3, 1x,e10.3,
     :                   1x,e10.3, 1x,e10.3, 
     :                   1x,e10.3, 1x,e10.3, 
     :               0p, 
     :                   7x,f6.1,  7x,f6.1, 
     :                   2x,f6.1,
     :               1p,
     :                  10x,e10.3,6x,e10.3, 6x,e10.3,
     :                  10x,e10.3,6x,e10.3, 6x,e10.3,
     :                  10x,e10.3,6x,e10.3, 6x,e10.3,
     :                  11x,e10.3,7x,e10.3, 7x,e10.3
     :                                                      )') 
     :      period(iper), 
     :      psa_1_ar(iper), psa_2_ar(iper), 
     :      psa_1_ar(iper)*psa2sd_factor, psa_2_ar(iper)*psa2sd_factor, 
     :      psa_larger_ar(iper), psa_gm_ar(iper),
     :      psa_larger_ar(iper)*psa2sd_factor, 
     :      psa_gm_ar(iper)*psa2sd_factor,
     :      psamaxrot(iper), psaminrot(iper),
     :      psamaxrot(iper)*psa2sd_factor, 
     :      psaminrot(iper)*psa2sd_factor,
     :      angle4psamax(iper), angle4psamin(iper),
     :      rot_angle(indx_min),
     :      gmean(iper,indx_min), gmean(iper,indx_min)/981.0, sd,
     :      gmrotd50(iper), gmrotd50(iper)/981.0, sd_gmrotd50,
     :      gmrotd00(iper), gmrotd00(iper)/981.0, sd_gmrotd00, 
     :      gmrotd100(iper), gmrotd100(iper)/981.0, sd_gmrotd100 
        end do

        close (nu_out)
        
        deallocate( a_in )
        deallocate( a_out )
        deallocate( vel, dis )
        deallocate( osc_ts_as_recorded )
        deallocate( osc_ts_rotated )
        deallocate( dum )
        
        call get_lun(nu_penalty)
        f_penalty = ' '
        f_penalty = f_in(1)(1:nc_f_in(1))//'.penalty'//
     :              f_ext(1:nc_f_ext)
        call trim_c(f_penalty,nc_f_penalty)
        open(nu_penalty,file=f_penalty(1:nc_f_penalty),
     :       status='unknown')

        write(nu_penalty,'(6x,a, 1x,a, 4x,a, 
     :                     1x,a, 5x,a, 8x,a)') 
     :              'per', 'rot_angle', 'penalty', 
     :              'rot_angle_min', 'psa/gmrot', 'gmean'

        do iper = 1, nperiods
          do iz = 1, nrot 
          
 
            write(nu_penalty,'(1x,f8.4, 2x,f8.3, 1p, 1x,e10.3,
     :                         0p, 7x,f7.2, 1p, 2(1x,e12.5))')
     :        period(iper), rot_angle(iz), penalty(iz),
     :        rot_angle(indx_min),  
     :                      gmean_norm(iper,iz), gmean(iper,iz) 
           end do
           write(nu_penalty,*)          
        end do
        
        close(nu_penalty)
        
      
        call date_and_time(values=idate_time_values)
        time_sec_stop = float(idate_time_values(5)*60*60+
     :                           idate_time_values(6)*60   +
     :                           idate_time_values(7))     +
     :                    float(idate_time_values(8))/1000.0
     
       write(nu_sum,'(a,1x,f7.3)') ' calc time = ',
     :                     time_sec_stop - time_sec_start
     
      go to 1000    ! loop back for another pair of components

      end
! ---------------------------------------------------------------- SMC2PSAGMRot

!      include '\forprogs\acc2vd.for'     
!      include '\forprogs\Trapezoidal_Integration.for'
!      include '\forprogs\rmvtrend.for'     
!      include '\forprogs\rotate.for'     
!      include '\forprogs\get_lun.for'     
!      include '\forprogs\skip.for'     
!      include '\forprogs\smcread.for'     
!      include '\forprogs\mnmaxidx.for'     
!      include '\forprogs\abs_mnmaxidx.for'     
!      include '\forprogs\trim_c.for'     
!      include '\forprogs\upstr.for'     
!      include '\forprogs\smc_npts.for'     
!      include '\forprogs\distaz.for'     
!      include '\forprogs\rc_subs.for'     
!      include '\forprogs\skipcmnt.for'     
!      include '\forprogs\rdrvaa_rd_ts.for'
!      include '\forprogs\rdcalcts.for'
!      include '\forprogs\moment.for'
!      include '\forprogs\select.for'

      include '\forprogs\smc2psa_util_subs.for'
  
  

!      subroutine RCF (KPARM,CSTR_IN,NCSTR_IN,F,ibgn,iend, ISTAT)

 
!Write a file with az1, daz, (period, gmean, gmean-%, gmean_50%, gmean+%), repeated for a set of periods specficied in input  


      
